#!/usr/bin/env python
##
##   Copyright (C) 2007 Mehdi Bayazee <Bayazee@Gmail.com>
##   Copyright (C) 2009-2010 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
##
##   Iranian (Jalali) calendar: 
##           http://en.wikipedia.org/wiki/Iranian_calendar
##   Islamic (Hijri) calendar:
##           http://en.wikipedia.org/wiki/Islamic_calendar
##   Gregorian calendar:
##           http://en.wikipedia.org/wiki/Gregorian_calendar
##
##   This program is free software; you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation; either version 3 of the License,  or
##   (at your option) any later version.
##
##   This program is distributed in the hope that it will be useful, 
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
##   You should have received a copy of the GNU General Public License along
##   with this program. If not, see <http://www.gnu.org/licenses/gpl.txt>.

moduleNames = ('gregorian', 'jalali', 'hijri', 'julian', 'hebrew')
modNum = len(moduleNames)

DATE_JD			= -1 ## Julian Day!
DATE_GREG		= 0  ## Gregorian(common calendar)
DATE_JALALI		= 1  ## Jalali, Iranian
DATE_HIJRI		= 2  ## Arabic, Islamic
DATE_JULIAN		= 3  ## Julian Calendar
DATE_HEBREW		= 4  ## Jewish


from math import floor, ceil
import time, os, sys

def my_raise(File=None):
  i = sys.exc_info()
  if File==None:
    print('line %s: %s: %s'%(i[2].tb_lineno, i[0].__name__, i[1]))
  else:
    print('File %s line %s: %s: %s'%(File, i[2].tb_lineno, i[0].__name__, i[1]))

## jd ~= time.time()/(24*3600.0) + J1970
## time.time() ~= (jd-J1970)*(24*3600.0) 


from paths import *

## jwday: Calculate day of week from Julian day
jwday = lambda jd: (jd + 1) % 7


getWeekDay = lambda y, m, d, mode, firstDay=0: jwday(to_jd(y, m, d, mode)-firstDay)


sys.path.insert(0, modDir)
modules = []
for name in moduleNames:
  mod = __import__(name)
  for attr in ('name', 'desc', 'monthName', 'getMonthName', 'getMonthsInYear',
  'minMonthLen', 'maxMonthLen', 'options', 'save', 'to_jd', 'jd_to', 'getMonthLen'):
     if not hasattr(mod, attr):
       raise AttributeError('module "%s" has no attribute "%s"'%(name, attr))
  modules.append(mod)

jd_to = lambda jd, target: modules[target].jd_to(jd)

to_jd = lambda y, m, d, source: modules[source].to_jd(y, m, d)

def convert(y, m, d, source, target):
  return jd_to(to_jd(y, m, d, source), target)

getMonthLen = lambda year, month, mode: modules[mode].getMonthLen(year, month)

########################################################
def gmtime(sec=None, mode=DATE_GREG):
  if mode==DATE_GREG:
    return time.gmtime(sec)
  t = time.gmtime(sec)
  t[:3] = convert(t[0], t[1], t[2], DATE_GREG, mode)
  return t

def localtime(sec=None, mode=DATE_GREG):
  if mode==DATE_GREG:
    return list(time.localtime(sec))
  t = list(time.localtime(sec))
  t[:3] = convert(t[0], t[1], t[2], DATE_GREG, mode)
  return t

def mktime(t, mode=DATE_GREG):
  if mode==DATE_GREG:
    return time.mktime(t)
  t[:3] = convert(t[0], t[1], t[2], mode, DATE_GREG)
  return time.mktime(t)



