#!/usr/bin/python
## Copyright (C) 2010 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
## converted from libkal
## libkal: Copyright (C) 1996-1998 Petr Tomasek <tomasek@etf.cuni.cz>, under the GNU LGPL v2+

name = 'hebrew'
desc = 'Hebrew (Jewish)'
monthNameNorm=('Tishri', 'Cheshvan', 'Kislev', 'Tevet', 'Shevat', 'Adar', 'Nisan', 'Iyyar', 'Siwan', 'Tamuz', 'Av', 'Elul')
monthNameLeap=('Tishri', 'Cheshvan', 'Kislev', 'Tevet', 'Shevat', 'Adar', 'Adar II', 'Nisan', 'Iyyar', 'Siwan', 'Tamuz', 'Av', 'Elul')
monthName = monthNameNorm ## For compatibility

minMonthLen = 29
maxMonthLen = 31

options = ()

def save():
  pass

from math import floor
ifloor = lambda x: int(floor(x))

def prest(w2): ## int(int)
  if w2>0:
    return divmod(7*w2+1, 19)[0] - divmod(7*w2-6, 19)[0]
  else:
    return 0

def uuw(u2, u3, w2, y): ## int return (int u2,long u3,int w2,int y)
  uu = 0
  if u2 in (2, 4, 6):
    uu = 1
  elif u2==1:
    if u3>=16404 and prest(w2)==0:
      uu = 2
  elif u2==0:
    if u3>=23269 and prest(y-2 % 19)==1:
      uu = 1
  return uu


def rosh_hashana(y): ## long(int)
  (w1, w2) = divmod(y-1, 19)
  w = 235*w1 + 12*w2 + divmod(7*w2+1, 19)[0]
  (tmp, u3) = divmod(13753*w + 12084, 25920)
  uu = 29*w + tmp
  (u1, u2) = divmod(uu, 7)
  return 347998 + 7*u1 + u2 + uuw(u2, u3, w2, y)

  
def z_s(m, p, q): ## int(int, int, int)
  ss = (m-1)*29
  if p==0 and m>7:
    ss -= 29
  if m>6:
    ss += (divmod(m-7,2)[0] + 3)
  else:
    ss += divmod(m,2)[0]
  if p==1 and m>6:
    ss += 1
  if q==0 and m>3:
    ss -= 1
  if q==2 and m>2:
    ss += 1
  return ss

def to_jd(y, m, d): ## long(int, int, int)
  j1 = rosh_hashana(y)
  j2 = rosh_hashana(y+1)
  j = j2-j1
  if j>380:
    p = 1
    q = j-383 
  else:
    p = 0
    q = j-353
  return j1 + z_s(m, p, q) + d - 1

def jd_to(jd): ## long jd
  assert type(jd)==int
  ## ww=(long)((float)((jd-347998)/29.5305941));
  ww = floor((jd-347998)/29.5305941)
  (w1, tmp) = divmod(ww, 235)
  w2 = divmod(tmp, 12)[0]
  y = 19*w1 + w2 - 1
  jd2 = rosh_hashana(y)
  while True:
    jd1 = jd2
    y += 1
    jd2 = rosh_hashana(y)
    if jd2 > jd:
      break
  y -= 1
  j = jd2-jd1
  if j>380:
    p = 1
    q = j-383
  else:
    p = 0
    q = j-353
  j = jd - jd1
  m = 0
  while True:
    m += 1
    if j < z_s(m, p, q):
      break
  y = ifloor(y)
  m -=1 ## = int(m-1)
  if m==13:
    m = 1
    y += 1
  elif m==0:
    m = 12
    y -= 1
  d = int(j - z_s(m, p, q) + 1)
  #assert 1 <= m <= 12
  #assert d > 0
  #if not (1 <= m <= 12 and d > 0):
  #  print '\nBad hebrew output date %.4d/%.2d/%.2d\n'%(y, m, d)
  return (y, m, d)

## is the year leap?
def isLeap(y):
  ## Is the number of days
  ## in the year > 360 ?
  return rosh_hashana(y+1) - rosh_hashana(y) > 360

def getMonthsInYear(y):
  if y==None or isLeap(y):
    return 13
  else:
    return 12

def getMonthLen(y, m):
  if m==getMonthsInYear(y):
    return to_jd(y+1, 1, 1) - to_jd(y, m, 1)
  else:
    return to_jd(y, m+1, 1) - to_jd(y, m, 1)


def getMonthName(m, y):
  if y==None or isLeap(y):
    return monthNameLeap[m-1]
  else:
    return monthNameNorm[m-1]

def getMonthList(y):
  if y==None or isLeap(y):
    return monthNameLeap
  else:
    return monthNameNorm





