#!/usr/bin/env python
##
##   Copyright (C) 2007 Mehdi Bayazee <Bayazee@Gmail.com>
##   Copyright (C) 2009-2010 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
##
##
##   This program is free software; you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation; either version 3 of the License,  or
##   (at your option) any later version.
##
##   This program is distributed in the hope that it will be useful, 
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
##   You should have received a copy of the GNU General Public License along
##   with this program. If not, see <http://www.gnu.org/licenses/gpl.txt>.

name = 'hijri'
desc = 'Hijri (Islamic)'
monthName = ('Muharram','Safar','Rabia\' 1','Rabia\' 2','Jumada 1','Jumada 2','Rajab','Sha\'aban','Ramadan','Shawwal','Dhu\'l Qidah','Dhu\'l Hijjah')

getMonthName = lambda m, y: monthName.__getitem__(m-1)
getMonthsInYear = lambda y: 12

from math import floor, ceil
ifloor = lambda x: int(floor(x))

epoch = 1948439.5

hijriAlg = 0
hijriUseDB = True

options = (

('hijriAlg', list, 'Hijri Calculation Algorithm',
('Internal', 'ITL (idate command)', 'ITL (idate command) Umm_Alqura')),

('hijriUseDB',bool,'Use (Iranian) database for Hijri months length')
)


minMonthLen = 29
maxMonthLen = 30

hijriDbInitH = (1426,2,1) ; hijriDbInitJD = 2453441 ## load from file
hijriDbEndJD = hijriDbInitJD = 0 ## load from file
hijriMonthLen={} ## load and calc from file

import os
from os.path import dirname

from paths import *

## Here load user options(hijriAlg and hijriUseDB) from file
confPath = '%s/%s.conf'%(confDir, name)
if os.path.isfile(confPath):
  exec(file(confPath).read())

def save():## Here save user options(hijriAlg and hijriUseDB) to file
  file(confPath, 'w').write('hijriAlg=%s\nhijriUseDB=%s'%(hijriAlg, hijriUseDB))


#if cal.hijriAlg>0:#???????????
#  out = os.popen('idate').read()
#  if out=='':
#    print('Command "idate" not found! (make sure to install package "itools")\nUsing default(internal) Hijri algorithm...')
#    cal.hijriAlg = 0


def loadHijriDb(path='%s/hijri.db'%modDir):
  global hijriDbInitH, hijriMonthLen, hijriDbEndJD, hijriDbInitJD
  if not os.path.isfile(path):
    return
  hijriMonthLenY={}
  lines = file(path).read().split('\n')
  for line in lines[1:]:
    if line!='':
      parts = [int(p) for p in line.split(' ')]
      hijriMonthLenY[parts[0]] = parts[1:]
  parts = [int(p) for p in lines[0].split(' ')]
  hijriDbInitH = parts[:3]
  hijriDbEndJD = hijriDbInitJD = parts[3]
  for y in hijriMonthLenY.keys():
    lst = hijriMonthLenY[y]
    for m  in xrange(len(lst)):
      ml = lst[m]
      if ml!=0:
        hijriMonthLen[y*12+m]=ml
        hijriDbEndJD += ml

loadHijriDb()

## is_leap: Is a given year a leap year in the Islamic calendar ?
is_leap = lambda year: (((year * 11) + 14) % 30) < 11


def to_jd_c(year, month, day):
    return ifloor(day + ceil(29.5 * (month - 1)) + \
                  (year - 1) * 354               + \
                  floor((3 + (11 * year)) / 30)  + \
                  epoch - 1)

def to_jd(year, month, day):
  #assert 1 <= month <= 12
  #assert 1 <= day <= maxMonthLen
  if hijriUseDB and hijriAlg==0:
    ym = year*12 + month-1
    (y0, m0, d0) = hijriDbInitH
    ym0 = y0*12 + m0-1
    if not ym in hijriMonthLen.keys():
      return to_jd_c(year, month, day)
    print 'using db to calc jd'
    jd = hijriDbInitJD
    for ymi in range(ym0, ym):
      jd += hijriMonthLen[ymi]
    return jd + day - 1
  else:
    return to_jd_c(year, month, day)


def jd_to_hijri_idate(jd, umm_alqura=False):
  import os
  (y, m, d) = jd_to_gregorian(jd)
  fixed = str(y).zfill(4) + str(m).zfill(2) + str(d).zfill(2)
  cmd = 'idate --gregorian %s'%fixed
  if umm_alqura:
    cmd += ' --umm_alqura'
  output = os.popen(cmd).read()
  try:
    (hd, hm, hy) = output.split('\n')[3].split(':')[1].split('/')
    hy = hy.split(' ')[0]
  except:
    my_raise(__file__)
    raise OSError('command "idate" not found')
  return (ifloor(hy), ifloor(hm), ifloor(hd))


def jd_to(jd):
  assert type(jd)==int
  "JD_TO_HIJRI: Calculate Islamic date from Julian day"
  if hijriAlg==1:
    return jd_to_hijri_idate(jd, umm_alqura=False)
  elif hijriAlg==2:
    return jd_to_hijri_idate(jd, umm_alqura=True)
  ## Now hijriAlg==0
  if hijriUseDB:
    #jd = ifloor(jd)
    if hijriDbEndJD > jd >= hijriDbInitJD:
      #(yi, mi, di) = hijriDbInitH
      #ymi = yi*12 + mi
      (y, m, d) = hijriDbInitH
      ym = y*12 + m-1
      while jd > hijriDbInitJD:
        monthLen = hijriMonthLen[ym]
        if jd-monthLen > hijriDbInitJD:
          ym += 1
          jd -= monthLen
        elif d+jd-hijriDbInitJD > monthLen:
          ym += 1
          d = d + jd - hijriDbInitJD - monthLen
          jd = hijriDbInitJD
        else:
          d = d + jd - hijriDbInitJD
          jd = hijriDbInitJD
      (y, m) = divmod(ym, 12)
      m+=1
      return (y, m, d)
  ##jd = floor(jd) + 0.5
  year = ifloor(((30 * (jd - epoch)) + 10646) / 10631)
  month = int(min(12, ceil((jd - (29 + to_jd(year, 1, 1))) / 29.5) + 1))
  day = int(jd - to_jd(year, month, 1)) + 1
  return year, month, day


def getMonthLen(y, m):
  if m==12:
    return to_jd(y+1, 1, 1) - to_jd(y, 12, 1)
  else:
    return to_jd(y, m+1, 1) - to_jd(y, m, 1)


